CON
  _clkmode             = xtal1 + pll16x                                               'Standard clock mode * crystal frequency = 80 MHz
  _xinfreq             = 5_000_000

  LCD_pin  = 12
  LCD_baud = 19200

OBJ
   LCD       : "simple_serial"
     
PUB MAIN
  LCD.Init(-1,LCD_pin,LCD_baud)
  DisplayCLS

  InitializeFiles(@Files)
  
  Pause(16)
  Halt

PRI Pause(count) ' Eighths Of A Second
  waitcnt(cnt+(clkfreq/8)*count)

PRI Halt
  DisplayCLS
  DisplayStr(0,0,string("System Halt."))
  DisplayPage
  Pause(8)
  cogstop(0)           ' halts main cog

DAT
Files
  byte { "filename" , 0, byte,  
       } "temp1"    , 0,  $31, {
       } "temp23"   , 0,  $32, {
       } "temp456"  , 0,  $34, {
       } $FF

PRI InitializeFiles(FilesPointer) | pointer, entry, dataoffset, datasize

  datasize:=1
  pointer~
  repeat until byte[FilesPointer][pointer]==$FF   '                                                                   
    DisplayCLS                                    ' files = "temp1,temp23,temp456"                                    
                                                  '                                                                   
    DisplayHex(0,0,byte[FilesPointer][pointer],2) ' first pass = $74      second pass = $74          third pass = $6D 
    entry     :=FilesPointer[pointer]             '                                                                   
    dataoffset:=strsize(entry)+1                  '                                                                   
    DisplayStr(1, 0,entry                    )    ' first pass = "temp1"  second pass = unprintable  third pass = unprintable     
    DisplayDec(2, 5,pointer                  )    ' first pass = 0        second pass = 7            third pass = 9    
    DisplayDec(2,10,dataoffset               )    ' first pass = 6        second pass = 1            third pass = 1    
    DisplayHex(1,18,byte[entry][dataoffset],2)    ' first pass = $31      second pass = $B4          third pass = $B4    
                                                  '                                                                   
    DisplayPage                                   '                                                                   
    pause(32)                                     '                                                                   

    pointer+=dataoffset+datasize ' advance pointer to next filename (or $FF)

CON {{ Display Methods }}

VAR   
  byte DisplayPageA[80]

PRI DisplayStr(y,x,stringptr) | pointer
  pointer~
  repeat strsize(stringptr)
    DisplayPageA[(y*20)+x+pointer-1]:=byte[stringptr][pointer++]

PRI DisplayCLS | pointer
  bytefill(@DisplayPageA,32,80)
  DisplayPage

PRI DisplayChar(y,x,a)
  DisplayPageA[(y*20)+x]:=a

PRI DisplayHex(y,x,value, digits) | a
  a~
  value <<= (8 - digits) << 2
  repeat digits
    DisplayChar(y,x+a,lookupz((value <-= 4) & $F : "0".."9", "A".."F"))
    a++

PRI DisplayDec(y,x,value) | i, j,pos, count,negative
{{Send value as decimal characters.
  Parameter:
    value - byte, word, or long value to send as decimal characters.}}
  count~
  negative~
  j := value == NEGX                                                            'Check for max negative
  i := 1_000_000_000                                                            'Initialize divisor
  if value < 0
    value := ||(value+j)                                                        'If negative, make positive; adjust for max negative
    negative~~
  repeat pos from -9 to 0                                                                     'Loop for 10 digits
    if value => i                                                               
      DisplayChar(y,pos+x,value / i + "0" + j*(i == 1))                             'If non-zero digit, output digit; adjust for max negative
      value //= i                                                               'and digit from value
      result~~
      count++                                                                  'flag non-zero found
    elseif result or i == 1
      DisplayChar(y,pos+x,"0")                                                                 'If zero digit (or only digit) output it
    i /= 10
  if negative
    DisplayChar(y,x-count,"-")                                                                   'and output sign

PRI DisplayPage | pointer
LCD.tx(128)
repeat pointer from 0 to 79
  LCD.tx(DisplayPageA[pointer])
